//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2012 Bernhard Beschow <bbeschow@cs.tu-berlin.de>

#include "ParsingRunner.h"

namespace Marble
{

ParsingRunner::ParsingRunner( QObject *parent )
    : QObject( parent )
{
    // nothing to do
}

}

#include "moc_ParsingRunner.cpp"
