//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2014      Calin Cruceru   <crucerucalincristian@gmail.com
//

#ifndef MARBLE_SCENEGRAPHICSTYPES_H
#define MARBLE_SCENEGRAPHICSTYPES_H

namespace Marble
{

/**
 * @brief: This namespace includes all available graphic item names a user can
 * interact with using the annotate plugin.
 */

namespace SceneGraphicsTypes
{

extern const char SceneGraphicAreaAnnotation[];
extern const char SceneGraphicGroundOverlay[];
extern const char SceneGraphicTextAnnotation[];
extern const char SceneGraphicPolylineAnnotation[];

}

}

#endif

