//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2011      Dennis Nienhüser <nienhueser@kde.org>
//

#include "Writer.h"

namespace Marble
{

Writer::Writer( QObject* parent ) : QObject( parent )
{
    // nothing to do
}

}

#include "moc_Writer.cpp"
