//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2009      Bastian Holst <bastianholst@gmx.de
//

// Own
#include "StoragePolicy.h"

using namespace Marble;

StoragePolicy::StoragePolicy( QObject *parent )
    : QObject( parent )
{}

#include "moc_StoragePolicy.cpp"
