//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2009      Bastian Holst <bastianholst@gmx.de>
//


#ifndef WEATHERGLOBAL_H
#define WEATHERGLOBAL_H

// Configuration defaults
const bool showConditionDefault = true;
const bool showTemperatureDefault = true;
const bool showWindDirectionDefault = false;
const bool showWindSpeedDefault = false;

#endif // WEATHERGLOBAL_H
