//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2014      Calin Cruceru <crucerucalincristian@gmail.com
//


#include "SceneGraphicsTypes.h"


namespace Marble
{

namespace SceneGraphicsTypes
{
const char SceneGraphicAreaAnnotation[] = "SceneGraphicAreaAnnotation";
const char SceneGraphicGroundOverlay[] = "SceneGraphicGroundOverlay";
const char SceneGraphicTextAnnotation[] = "SceneGraphicTextAnnotation";
const char SceneGraphicPolylineAnnotation[] = "SceneGraphicPolylineAnnotation";
}

}
