//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2012 Illya Kovalevskyy <illya.kovalevskyy@gmail.com>
//

#include "MarbleColors.h"

namespace Marble
{
namespace Oxygen
{

QColor const woodBrown6 = QColor::fromRgb( 56, 37, 9 );
QColor const woodBrown5 = QColor::fromRgb( 87, 64, 30 );
QColor const woodBrown4 = QColor::fromRgb( 117, 81, 26 );
QColor const woodBrown3 = QColor::fromRgb( 143, 107, 50 );
QColor const woodBrown2 = QColor::fromRgb( 179, 146, 93 );
QColor const woodBrown1 = QColor::fromRgb( 222, 188, 133 );
QColor const brickRed6 = QColor::fromRgb( 156, 15, 15 );
QColor const brickRed5 = QColor::fromRgb( 191, 3, 3 );
QColor const brickRed4 = QColor::fromRgb( 226, 8, 0 );
QColor const brickRed3 = QColor::fromRgb( 232, 87, 82 );
QColor const brickRed2 = QColor::fromRgb( 240, 134, 130 );
QColor const brickRed1 = QColor::fromRgb( 249, 204, 202 );
QColor const raspberryPink6 = QColor::fromRgb( 156, 15, 86 );
QColor const raspberryPink5 = QColor::fromRgb( 191, 3, 97 );
QColor const raspberryPink4 = QColor::fromRgb( 226, 0, 113 );
QColor const raspberryPink3 = QColor::fromRgb( 232, 82, 144 );
QColor const raspberryPink2 = QColor::fromRgb( 240, 130, 176 );
QColor const raspberryPink1 = QColor::fromRgb( 249, 202, 222 );
QColor const burgundyPurple6 = QColor::fromRgb( 106, 0, 86 );
QColor const burgundyPurple5 = QColor::fromRgb( 133, 2, 108 );
QColor const burgundyPurple4 = QColor::fromRgb( 160, 39, 134 );
QColor const burgundyPurple3 = QColor::fromRgb( 177, 79, 154 );
QColor const burgundyPurple2 = QColor::fromRgb( 193, 115, 176 );
QColor const burgundyPurple1 = QColor::fromRgb( 232, 183, 215 );
QColor const grapeViolet6 = QColor::fromRgb( 29, 10, 85 );
QColor const grapeViolet5 = QColor::fromRgb( 52, 23, 110 );
QColor const grapeViolet4 = QColor::fromRgb( 70, 40, 134 );
QColor const grapeViolet3 = QColor::fromRgb( 100, 74, 155 );
QColor const grapeViolet2 = QColor::fromRgb( 142, 121, 165 );
QColor const grapeViolet1 = QColor::fromRgb( 195, 180, 218 );
QColor const skyBlue6 = QColor::fromRgb( 0, 49, 110 );
QColor const skyBlue5 = QColor::fromRgb( 0, 67, 138 );
QColor const skyBlue4 = QColor::fromRgb( 0, 87, 174 );
QColor const skyBlue3 = QColor::fromRgb( 44, 114, 199 );
QColor const skyBlue2 = QColor::fromRgb( 97, 147, 207 );
QColor const skyBlue1 = QColor::fromRgb( 164, 192, 228 );
QColor const seaBlue6 = QColor::fromRgb( 0, 72, 77 );
QColor const seaBlue5 = QColor::fromRgb( 0, 96, 102 );
QColor const seaBlue4 = QColor::fromRgb( 0, 120, 128 );
QColor const seaBlue3 = QColor::fromRgb( 0, 167, 179 );
QColor const seaBlue2 = QColor::fromRgb( 0, 196, 204 );
QColor const seaBlue1 = QColor::fromRgb( 168, 221, 224 );
QColor const emeraldGreen6 = QColor::fromRgb( 0, 88, 63 );
QColor const emeraldGreen5 = QColor::fromRgb( 0, 115, 77 );
QColor const emeraldGreen4 = QColor::fromRgb( 0, 153, 102 );
QColor const emeraldGreen3 = QColor::fromRgb( 0, 179, 119 );
QColor const emeraldGreen2 = QColor::fromRgb( 0, 204, 136 );
QColor const emeraldGreen1 = QColor::fromRgb( 153, 220, 198 );
QColor const forestGreen6 = QColor::fromRgb( 0, 110, 41 );
QColor const forestGreen5 = QColor::fromRgb( 0, 137, 44 );
QColor const forestGreen4 = QColor::fromRgb( 55, 164, 44 );
QColor const forestGreen3 = QColor::fromRgb( 119, 183, 83 );
QColor const forestGreen2 = QColor::fromRgb( 177, 210, 143 );
QColor const forestGreen1 = QColor::fromRgb( 216, 232, 194 );
QColor const sunYellow6 = QColor::fromRgb( 227, 173, 0 );
QColor const sunYellow5 = QColor::fromRgb( 243, 195, 0 );
QColor const sunYellow4 = QColor::fromRgb( 255, 221, 0 );
QColor const sunYellow3 = QColor::fromRgb( 255, 235, 85 );
QColor const sunYellow2 = QColor::fromRgb( 255, 242, 153 );
QColor const sunYellow1 = QColor::fromRgb( 255, 246, 200 );
QColor const hotOrange6 = QColor::fromRgb( 172, 67, 17 );
QColor const hotOrange5 = QColor::fromRgb( 207, 73, 19 );
QColor const hotOrange4 = QColor::fromRgb( 235, 115, 49 );
QColor const hotOrange3 = QColor::fromRgb( 242, 155, 104 );
QColor const hotOrange2 = QColor::fromRgb( 242, 187, 136 );
QColor const hotOrange1 = QColor::fromRgb( 255, 217, 176 );
QColor const aluminumGray6 = QColor::fromRgb( 46, 52, 54 );
QColor const aluminumGray5 = QColor::fromRgb( 85, 87, 83 );
QColor const aluminumGray4 = QColor::fromRgb( 136, 138, 133 );
QColor const aluminumGray3 = QColor::fromRgb( 186, 189, 182 );
QColor const aluminumGray2 = QColor::fromRgb( 211, 215, 207 );
QColor const aluminumGray1 = QColor::fromRgb( 238, 238, 236 );

}

}
