//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2014 Sanjiban Bairagya <sanjiban22393@gmail.com>
//

#include "PlaybackItem.h"

namespace Marble
{

PlaybackItem::~PlaybackItem()
{
    //nothing to do
}

}

#include "moc_PlaybackItem.cpp"
