//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2015      Gábor Péterffy   <gabor.peterffy@gmail.com>
//

#include "NullMarbleWebView.h"

MarbleWebView::MarbleWebView(QWidget *parent):
    QWidget(parent)
{
}

void MarbleWebView::setUrl(const QUrl &url)
{

}
