//
// This file is part of the Marble Virtual Globe.
//
// This program is free software licensed under the GNU LGPL. You can
// find a copy of this license in LICENSE.txt in the top directory of
// the source code.
//
// Copyright 2007      Christian Ehrlicher <ch.ehrlicher@gmx.de>
// Copyright 2008      Patrick Spendrin <ps_ml@gmx.de>
//

#ifndef GEODATA_EXPORT_H
#define GEODATA_EXPORT_H

#include "marble_export.h"

// please make sure to create a real export-header if geodata becomes an independent library
#define GEODATA_EXPORT MARBLE_EXPORT
#define GEODATA_DEPRECATED MARBLE_DEPRECATED

#endif // GEODATA_EXPORT_H
